/*
 * cfl - attempts to convert Consolidated Frequency List to data files
 *       that can be used with freqgen
 *
 * Copyright 2001 by Fred Walter
 * Permission is herein granted:
 *    to freely use this software for non-commercial use
 * You may *not* include this software in anything that you sell unless
 * you license the software from me.
 *
 * Email:
 *    fred_walter@hotmail.com
 *
 * Usage:
 *    cfl cfl_file
 *
 * cfl puts the resulting data files in the "data" subdirectory.
 *
 * The Consolidated Frequency List is located at: 
 *    http://www.mindspring.com/~turf/alt/elec/cfl.htm
 * Use your browser to "Select All" text on this page,
 * and then "Copy" your selection to the clipboard. 
 * Then start up notepad and "Paste" the text to an empty document,
 * and save the document with a reasonable name (eg. cfl.txt).
 * Run "cfl cfl.txt" to generate data files that can be used with freqgen.
 *
 * Operating Systems:
 *    Windows 95, Windows 98, Windows ME, Windows NT, Windows XP, etc.etc.etc.
 *
 * Compilation Instructions:
 *    Microsoft Visual C++:
 *       cl -Zi cfl.c
 *
 * Revision history:
 *    May 1, 2001
 *         1.0 - first public release
 *
 * TODO:
 *    - add "-d duration" option to set the default duration
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#undef BUFSIZ
#define BUFSIZ 2048

int
main(int argc, char *argv[])
{
	char errline[BUFSIZ];
	char line[BUFSIZ];
	FILE *fi;

	if (argc != 2) {
		fprintf(stderr, "Usage: cfl_file\n");
		exit(2);
	}

	fi = fopen(argv[1], "r");
	if (fi == NULL) {
		perror(argv[1]);
		exit(1);
	}

	(void) mkdir("data");

	while (fgets(line, sizeof line, fi)) {
		char *p1;
		char *data;
		char *comment;
		FILE *fo;
		char filename[BUFSIZ];

		p1 = strchr(line, '\n');
		if (p1 != NULL)
			*p1 = '\0';

		if (line[0] == '\0')
			continue;
		if (strncmp(line, "The Consolidated", 16) == 0)
			continue;
		if (strncmp(line, "Compiled by turf", 16) == 0)
			continue;
		if (strncmp(line, "Notes:", 6) == 0)
			continue;

		strcpy(errline, line); /* for error messages */

		p1 = strstr(line, " - ");
		if (p1 == NULL)
			p1 = strstr(line, " -");
		if (p1 == NULL) {
			fprintf(stderr, "bad format: %s\n", errline);
			continue;
		}
		*p1 = '\0';
		data = p1+2;

		comment = NULL;
		p1 = strchr(line, '(');
		if (p1) {
			comment = p1+1;
			while (p1 > line) {
				*p1 = '\0';
				p1--;
				if (*p1 != ' ' && *p1 != '\t')
					break;
			}
			p1 = strchr(comment, '\0');
			while (p1 > comment) {
				p1--;
				if (*p1 == ')' || *p1 == ' ' || *p1 == '\t')
					*p1 = '\0';
				else
					break;
			}
		}

		p1 = strchr(line, '"');
		while (p1 != NULL) {
			*p1 = '+';
			p1 = strchr(line, '"');
		}
		p1 = strchr(line, ')');
		while (p1 != NULL) {
			*p1 = '_';
			p1 = strchr(line, '"');
		}
		p1 = strchr(line, '/');
		while (p1 != NULL) {
			*p1 = '+';
			p1 = strchr(line, '/');
		}
		p1 = strchr(line, '\\');
		while (p1 != NULL) {
			*p1 = '+';
			p1 = strchr(line, '\\');
		}
		p1 = strchr(line, '\t');
		while (p1 != NULL) {
			*p1 = '+';
			p1 = strchr(line, '\t');
		}
		p1 = strchr(line, ' ');
		while (p1 != NULL) {
			*p1 = '_';
			p1 = strchr(line, ' ');
		}
		p1 = strchr(line, '\'');
		while (p1 != NULL) {
			memmove(p1, p1+1, strlen(p1));
			p1 = strchr(line, '\'');
		}
		while (*line == '_')
			memmove(line, line+1, strlen(line));
		p1 = strchr(line, '\0');
		while (--p1 >= line) {
			if (*p1 == '_')
				*p1 = '\0';
			else
				break;
		}
		for(p1 = line; p1; p1 = strchr(p1+1, '_')) {
			while (p1[0] == '_' && p1[1] == '_')
				memmove(p1, p1+1, strlen(p1));
		}
		if (*line == '\0') {
			fprintf(stderr, "bad format: %s\n", errline);
			continue;
		}
		sprintf(filename, "data/%s", line);

		fo = fopen(filename, "a");
		if (fo == NULL) {
			perror(filename);
			continue;
		}
		if (comment)
			fprintf(fo, "# %s\n", comment);
		p1 = data;
		for(;;) { 
			char *next_number;
			char *p3;
			char c;

			while (isspace(*p1))
				p1++;
			if (*p1 == '\0')
				break;
			if (!isdigit(*p1)) {
				fprintf(stderr, "bad format: %s\n", errline);
				fclose(fo);
				(void)unlink(filename);
				break;
			}
			next_number = strchr(p1, ',');
			if (next_number == NULL)
				next_number = strchr(p1, '\0');
			c = *next_number;

			data = next_number;
			*data = '\0';
			while (data > p1 && (isspace(*data) || *data == '\0'))
				*data-- = '\0';

			p3 = strchr(p1, ' ');
			if (p3 == NULL) {
				fprintf(fo, "%s, 180\n", p1);
			} else if (strncmp(p3, " for ", 5) != 0) {
				fprintf(stderr, "bad format: mangled for: %s\n",
					errline);
				fclose(fo);
				(void)unlink(filename);
				break;
			} else {
				*p3 = '\0';
				p3 += 5;
				while (isspace(*p3))
					p3++;

				fprintf(fo, "%s, %s\n", p1, p3);
			}
			*next_number = c;
			p1 = next_number;
			if (*p1 == '\0')
				break;
			p1++;
		}
		(void) fclose(fo);
	}
}
